#!/usr/bin/env python
 
import urllib, urllib2, urlparse, string, sys, os
import ClientCookie
import MySQLdb
from rsconfig import RSConfig

class Processor:
    def __init__(self):
        """Create class for downloading
        """
        self.config = RSConfig()

    def _getfilename(self, response):
        """Extract filename from response
        """
        filename = response.info().get('Content-Disposition')
        
        if filename is not None:
            idx = string.find(filename, "filename=")
            if (idx < 0):
                filename = None
                            
        if filename is not None:
            filename = filename[(idx + 9):].strip()
            print 'filename from content-disposition =', filename
        else:
            filename = response.geturl()
            idx = string.rfind(filename, "/")
            filename = filename[(idx + 1):].strip()
            print 'filename from url =', filename
            
        if (filename[0] == '"' and filename[-1] == '"'):
            filename = filename[1:len(filename)-1]

        return filename
        
    def _getfilesize(self, filename):
        """Get file size
        """
        return os.stat(filename).st_size;

    def _download_file(self, response):
        """Download file from response object
        """
        remote_file = self._getfilename(response);

        try:
            local_file = open(self.config.folder + remote_file, "wb")
            while 1:
                data = response.read(self.config.bytes)
                if (not data):
                    break
                local_file.write(data)
                
            local_file.close()
            
        except IOError:
            print "Error saving file!", sys.exc_info()[0]
            print sys.exc_info()[1]
            return -100, 0, remote_file
        except:
            print "Error saving file!", sys.exc_info()[0]
            return -101, 0, remote_file

        size = self._getfilesize(remote_file);
        
        return 0, size, remote_file
        
    def _download_with_login(self, url, data, fileurl):
        """Download file and login before the process
        """

        # connect to rapidshare and create cookie
        request = ClientCookie.Request(url)
        request.add_header('User-agent', 'Mozilla/5.0')
        request.add_data(urllib.urlencode(data))
        try:
            response = ClientCookie.urlopen(request)
        except urllib2.HTTPError:
            print "Login url does not exist!"
            return -1, 0, None
        except:
            print "Error opening connection to login!"
            return -2, 0, None

        # download the file
        request2 = ClientCookie.Request(fileurl)
        request2.add_header('User-agent', 'Mozilla/5.0')
        
        try:
            response2 = ClientCookie.urlopen(request2)
        except urllib2.HTTPError:
            print "File does not exist on server!"
            return -3, 0, None
        except:
            print "Error opening connection to file!"
            return -4, 0, None

        return self._download_file(response2)
        
    def _download_rs(self, fileurl):
        """Download from rapidshare
        """

        # basic data for rapidshare
        url = 'https://ssl.rapidshare.com/cgi-bin/premiumzone.cgi'  
        data = {'login': self.config.rs_login, 'password': self.config.rs_password} 

        return self._download_with_login(url, data, fileurl)

    def _download_nl(self, fileurl):
        """Download from rapidshare
        """

        # basic data for Netload
        url = 'http://netload.in/index.php'  
        data = {'txtuser': self.config.nl_login, 'txtpass': self.config.nl_password, 'txtcheck' : 'login', 'txtlogin' : ''} 

        return self._download_with_login(url, data, fileurl)
        
    def download(self, fileurl):
        """Download from provided link
        """        
        fileurl_up = string.upper(fileurl)
        if (fileurl_up[:22] == 'HTTP://RAPIDSHARE.COM/'):
            #fileurl = fileurl[:22] + urllib.quote(fileurl[23:])  
            return self._download_rs(fileurl)
        if (fileurl_up[:18] == 'HTTP://NETLOAD.IN/'):
            #fileurl = fileurl[:18] + urllib.quote(fileurl[19:])  
            return self._download_nl(fileurl)
            
    def _update_status(self, link, status, filename=None, filesize=None):
        """Update status of link processing
        """
        db = MySQLdb.connect(host=self.config.mysql_host, user=self.config.mysql_user, passwd=self.config.mysql_password, db=self.config.mysql_database)
        db.autocommit(1)
        cursor = db.cursor()
        if filename is not None:            
            if (filesize is not None):
                cursor.execute("UPDATE " + self.config.mysql_database + ".jobs SET processed=now(), filename=%s, size=%s, state=%s WHERE link=%s", (filename, filesize, status, link))
            else:
                cursor.execute("UPDATE " + self.config.mysql_database + ".jobs SET processed=now(), filename=%s, state=%s WHERE link=%s", (filename, status, link))
        else:
            cursor.execute("UPDATE " + self.config.mysql_database + ".jobs SET state=%s WHERE link=%s", (status, link))
        db.commit()
        cursor.close()
        db.close()
        
    def _process_download(self):    
        """Download links in queu
        """
        # get links to save
        db = MySQLdb.connect(host=self.config.mysql_host, user=self.config.mysql_user, passwd=self.config.mysql_password, db=self.config.mysql_database)
        db.query("select link from " + self.config.mysql_database + ".jobs where state = 0 order by inserted, link")
        result = db.store_result()
        
        # process links one by one
        for row in result.fetch_row():
            # download the file
            self._update_status(row[0], 1)
            status, size, filename = self.download(row[0])
            
            # if status is OK then store filename
            if (status == 0):
                self._update_status(row[0], 2, filename, size)
            else:
                self._update_status(row[0], 3)

        db.close()
                
    def _process_remove(self):
        """Remove links in queu
        """
        db = MySQLdb.connect(host=self.config.mysql_host, user=self.config.mysql_user, passwd=self.config.mysql_password, db=self.config.mysql_database)
        db.query("select filename, link from " + self.config.mysql_database + ".jobs where state = -1 order by inserted, link")
        result = db.store_result()

        #remove files
        db.autocommit(1)
        cursor = db.cursor()
        tuples = []
        for row in result.fetch_row():
            tuples += [(row[1])]
            if (row[0] is not None and row[0] != ""):
                try:
                    os.remove(self.config.folder + row[0])
                except:
                    print "Unable to remove file"

        # delete them from db
        cursor.executemany("DELETE FROM " + self.config.mysql_database + ".jobs WHERE link=%s AND state=-1", tuples)
        db.commit()
        cursor.close()
        db.close()
            
    def _process_redownload(self):
        """Redownload links in queu
        """
        db = MySQLdb.connect(host=self.config.mysql_host, user=self.config.mysql_user, passwd=self.config.mysql_password, db=self.config.mysql_database)
        db.query("select filename, link from " + self.config.mysql_database + ".jobs where state = -2 order by inserted, link")
        result = db.store_result()

        #remove files
        db.autocommit(1)
        cursor = db.cursor()
        tuples = []
        for row in result.fetch_row():
            tuples += [(row[1])]
            if (row[0] is not None and row[0] != ""):
                try:
                    os.remove(self.config.folder + row[0])
                except:
                    print "Unable to remove file"

        # set to download
        cursor.executemany("UPDATE " + self.config.mysql_database + ".jobs SET inserted = now(), state = 0 WHERE link=%s AND state=-2", tuples)
        db.commit()
        cursor.close()
        db.close()
            
    def process(self):
        """Process links
        """
        
        self._process_remove()
        self._process_redownload()
        self._process_download()
    

processor = Processor()
processor.process()
